/****************************************************************************
** Meta object code from reading C++ file 'MessageDb.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.15.0/src/MessageDb.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MessageDb.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9MessageDbE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageDb::qt_create_metaobjectdata<qt_meta_tag_ZN9MessageDbE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageDb",
        "messageAdded",
        "",
        "Message",
        "msg",
        "MessageOrigin",
        "origin",
        "messageUpdated",
        "message",
        "messagesRemoved",
        "accountJid",
        "chatJid",
        "messageRemoved",
        "newLastMessage",
        "draftMessageAdded",
        "draftMessageUpdated",
        "draftMessageRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'messageAdded'
        QtMocHelpers::SignalData<void(const Message &, MessageOrigin)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'messageUpdated'
        QtMocHelpers::SignalData<void(const Message &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Signal 'messagesRemoved'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'messageRemoved'
        QtMocHelpers::SignalData<void(const Message &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 },
        }}),
        // Signal 'draftMessageAdded'
        QtMocHelpers::SignalData<void(const Message &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'draftMessageUpdated'
        QtMocHelpers::SignalData<void(const Message &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'draftMessageRemoved'
        QtMocHelpers::SignalData<void(const Message &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageDb, qt_meta_tag_ZN9MessageDbE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageDb::staticMetaObject = { {
    QMetaObject::SuperData::link<DatabaseComponent::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MessageDbE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MessageDbE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9MessageDbE_t>.metaTypes,
    nullptr
} };

void MessageDb::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MessageDb *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->messageAdded((*reinterpret_cast<std::add_pointer_t<Message>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MessageOrigin>>(_a[2]))); break;
        case 1: _t->messageUpdated((*reinterpret_cast<std::add_pointer_t<Message>>(_a[1]))); break;
        case 2: _t->messagesRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->messageRemoved((*reinterpret_cast<std::add_pointer_t<Message>>(_a[1]))); break;
        case 4: _t->draftMessageAdded((*reinterpret_cast<std::add_pointer_t<Message>>(_a[1]))); break;
        case 5: _t->draftMessageUpdated((*reinterpret_cast<std::add_pointer_t<Message>>(_a[1]))); break;
        case 6: _t->draftMessageRemoved((*reinterpret_cast<std::add_pointer_t<Message>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Message >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageOrigin >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Message >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Message >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Message >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Message >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Message >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const Message & , MessageOrigin )>(_a, &MessageDb::messageAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const Message & )>(_a, &MessageDb::messageUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const QString & , const QString & )>(_a, &MessageDb::messagesRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const Message & )>(_a, &MessageDb::messageRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const Message & )>(_a, &MessageDb::draftMessageAdded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const Message & )>(_a, &MessageDb::draftMessageUpdated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const Message & )>(_a, &MessageDb::draftMessageRemoved, 6))
            return;
    }
}

const QMetaObject *MessageDb::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageDb::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MessageDbE_t>.strings))
        return static_cast<void*>(this);
    return DatabaseComponent::qt_metacast(_clname);
}

int MessageDb::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DatabaseComponent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void MessageDb::messageAdded(const Message & _t1, MessageOrigin _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void MessageDb::messageUpdated(const Message & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MessageDb::messagesRemoved(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void MessageDb::messageRemoved(const Message & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MessageDb::draftMessageAdded(const Message & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MessageDb::draftMessageUpdated(const Message & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MessageDb::draftMessageRemoved(const Message & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
